@@ -20,8 +20,7 @@ Requires HXT.
 
 module Main where
 
-import Text.XML.HXT.Arrow
-import Text.XML.HXT.Arrow.Edit
+import Text.XML.HXT.Core
 import Control.Arrow
 import Control.Arrow.ArrowList
 import Control.Monad (liftM)
@@ -126,7 +125,7 @@ processOneFile src = do
   [syntax] <- runX $ application src
   let name = nameFromPath src
   let outFile = joinPath [libraryPath, "Syntax", addExtension name "hs"]
-  let includeLangs = nub $ map (drop 2 . parserContext) $
+  let includeLangs = nub $ filter (/= name) $ map (drop 2 . parserContext) $
         filter (\p -> (parserType p) == "IncludeRules" && "##" `isPrefixOf` (parserContext p)) $ 
         concatMap contParsers $ synContexts syntax
   let includeImports = map (("import qualified " ++) . langNameToModule) includeLangs
@@ -225,15 +224,16 @@ mkParser syntax =
                             \    Left err     -> Left $ show err\n\
                             \    Right result -> Right result"
       endLineParser = text "pEndLine = do" $$
-                      (nest 2 $ text "newline <|> (eof >> return '\\n')" $$
+                      (nest 2 $ text "lookAhead $ newline <|> (eof >> return '\\n')" $$
                                 text "context <- currentContext" $$
                                 text "case context of" $$
                                 (nest 2 $ (vcat $ map (\cont -> text (show $ contName cont) <> text " -> " <> 
-                                            switchContext (contLineEndContext cont)) $ synContexts syntax) $$
-                                          (text $ "_ -> return ()")) $$
+                                            switchContext (contLineEndContext cont) <>
+                                            if "#pop" `isPrefixOf` (contLineEndContext cont)
+                                               then text " >> pEndLine"
+                                               else text " >> pHandleEndLine") $ synContexts syntax) $$
+                                          (text $ "_ -> pHandleEndLine")))
                                 {- text "pushContext (fromMaybe \"#stay\" $ lookup context lineBeginContexts)" $$ -}
-                                text "lineContents <- lookAhead wholeLine" $$
-                                text "updateState $ \\st -> st { synStCurrentLine = lineContents, synStCharsParsedInLine = 0, synStPrevChar = '\\n' }")
       -- we use 'words "blah blah2 blah3"' to keep ghc from inlining the list, which makes compiling take a long time
       listDef (n, list) = text $ listName n ++ " = Set.fromList $ words $ " ++
                                show (if keywordCaseSensitive (synKeywordAttr syntax)
@@ -328,20 +328,23 @@ mkSyntaxParser syntax context parser =
 
 switchContext next =
   case next of
-     x | "#pop" `isPrefixOf` x -> char '(' <> text (concat $ intersperse " >> " $ replicate (length (filter (=='#') x)) "popContext") <> 
-                                  text " >> return ())"
+     x | "#pop" `isPrefixOf` x -> char '(' <>
+          text (concat $ intersperse " >> " $ replicate (length (filter (=='#') x)) "popContext") <> char ')'
      "#stay" -> text "return ()"
      x -> text ("pushContext " ++ show x) 
 
 langNameToModule str =  "Text.Highlighting.Kate.Syntax." ++
   case str of
     "Alerts" -> "Alert"
+    "Alerts_indent" -> "Alert_indent"
     "C++" -> "Cpp"
     "CSS" -> "Css"
     "Doxygen" -> "Doxygen"
     "HTML" -> "Html"
     "Javadoc" -> "Javadoc"
     "JavaScript" -> "Javascript"
+    "SQL (MySQL)" -> "SqlMysql"
+    "DoxygenLua" -> "Doxygenlua"
     x -> x
 
 listName :: String -> String
@@ -367,7 +370,7 @@ nameFromPath = concat . map capitalize . words .
 
 application :: String -> IOSArrow b SyntaxDefinition
 application src
-    = readDocument [(a_validate, v_0)] src
+    = readDocument [withValidate no, withInputEncoding utf8] src
       >>>
       multi (hasName "language")
       >>>
